; Revised 17 October 2005 to accommodate 1/1/2006 leap second and 1/1/1999 leap second - DJB
;; Revised 16 November 2010 to accommodate 1/1/2009 - DEB
;;;Revised 07/02/2012 to accomadtae 6/30/2012 leap second- DEB
;;; Revised 11/06/2012 corrected 2012 revision and corrected -441849619 to -441849609  
;;; Revised 04/22/2016 to accomadtae 6/30/2015 leap second- DEB
;;; Revised 01/31/2017 to add 12/31/2016 leap second- DEB

;;;;;;;;;;;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;;;;;;;;Note: When adding leap seconds
;;;;;;;;Remember to increment LeapSeconds_TAI93_offset array !!!!!!!!!!!

function New_DateTime_Structure, num_elements
	aUTC = {year:fix(-99),month:fix(-99),day:fix(-99),hour:fix(-99), $
		minute:fix(-99),second:double(-99),doy:fix(-99)}
	aDateTime = {TAI93:double(-9e9), GPS:double(-99), UTC:replicate(aUTC,1), $
		       Local:replicate(aUTC,1)}
	newDateTime = replicate(aDateTime,num_elements)
	return, newDateTime
end


pro Calc_DateTime, thisDateTime, LONGITUDE=longitude

	sz = size(thisDateTime)
	len = sz(1)
	if keyword_set(longitude) then begin
		lsz = size(longitude)
		if (lsz(0) eq 0) then begin
			llen = 1
		endif else begin
			llen = lsz(1)
		endelse
		if (llen ne len and llen eq 1) then begin
			inlongitude = longitude
			longitude = replicate(inlongitude,len)
		endif else begin
			if (llen ne len) then begin
				print,'You have passed in a longitude array that does not match the date/time array length!'
				stop
			endif   
		endelse
	endif
	
	for i = long(0), long(len)-1, long(1) do begin
	
		DateTime = thisDateTime(i)
		
		if keyword_set(longitude) then begin
			uselongitude = longitude(i)
		endif
		
		; Do we have TAI93?
		
		if (DateTime.TAI93 gt -8e9) then begin
		
			; We accept TAI93 as gospel truth if present and overwrite any UTC info
			
			TAI93_to_UTC,DateTime
			
			; GPS is a constant conversion
			
			DateTime.GPS = DateTime.TAI93 + 409881608.0D
			
			; Was a longitude passed in?  If so, fill in the local time
			
			if keyword_set(longitude) then begin
				sec_offset = (uselongitude/360.0D)*86400.0D
				TempDT = New_DateTime_Structure(1)
				TempDT.TAI93 = DateTime.TAI93+sec_offset
				TAI93_to_UTC,TempDT
				DateTime.Local.year = TempDT.UTC.year
				DateTime.Local.month = TempDT.UTC.month
				DateTime.Local.day = TempDT.UTC.day
				DateTime.Local.doy = TempDT.UTC.doy
				DateTime.Local.hour = TempDT.UTC.hour
				DateTime.Local.minute = TempDT.UTC.minute
				DateTime.Local.second = TempDT.UTC.second
			endif
			
		endif else begin
		
			complete_ymd  =  ((DateTime.UTC.year gt -99) and (DateTime.UTC.month gt -99) and $
			    		  (DateTime.UTC.day gt -99))
			complete_ydoy =  ((DateTime.UTC.year gt -99) and (DateTime.UTC.doy gt -99))
			complete_date =  (complete_ymd and complete_ydoy)
			complete_time =  ((DateTime.UTC.hour gt -99) and (DateTime.UTC.minute gt -99) and $
					  (DateTime.UTC.second gt -99))
			
			; Must be a UTC date and/or time... Is date incomplete?
			
			if (not complete_date) then begin
			    
			    ; Incomplete.  Work on date first.  Is it in Y/M/D format w/o DOY?
			   		   
			   if (complete_ymd and not complete_ydoy) then begin
			        
			        ; Fill in the day of year
			        
			        YMD_to_DOY, DateTime

				complete_ymd  =  ((DateTime.UTC.year gt -99) and (DateTime.UTC.month gt -99) and $
				    		  (DateTime.UTC.day gt -99))
				complete_ydoy =  ((DateTime.UTC.year gt -99) and (DateTime.UTC.doy gt -99))
				complete_date =  (complete_ymd and complete_ydoy)				        
							        
			    endif else begin
			    
			    	; Is it in Y/DOY w/o M,D?
			    	
			    	if (complete_ydoy and not complete_ymd) then begin
			    	    	
			    	    	; Fill in M,D
			    	    	
			    	    	DOY_to_MD, DateTime

					complete_ymd  =  ((DateTime.UTC.year gt -99) and (DateTime.UTC.month gt -99) and $
					    		  (DateTime.UTC.day gt -99))
					complete_ydoy =  ((DateTime.UTC.year gt -99) and (DateTime.UTC.doy gt -99))
					complete_date =  (complete_ymd and complete_ydoy)				        
			    	    	
			    	endif else begin
			    	
			    		; Date is incomplete, can't do anything else in this routine.
			    		
			    	endelse
			    	
			    endelse
			    
			endif
			
			; Is it now a complete date and time?  If so, fill in the TAI93, GPS and possibly local times.
			
			if (complete_date and complete_time) then begin
			
				UTC_to_TAI93, DateTime    
				DateTime.GPS = DateTime.TAI93 + 409881608.0D
			
				; Was a longitude passed in?  If so, fill in the local time
				
				if keyword_set(longitude) then begin
					sec_offset = (uselongitude/360.0D)*86400.0D
					TempDT = New_DateTime_Structure(1)
					TempDT.TAI93 = DateTime.TAI93+sec_offset
					TAI93_to_UTC,TempDT
					DateTime.Local.year = TempDT.UTC.year
					DateTime.Local.month = TempDT.UTC.month
					DateTime.Local.day = TempDT.UTC.day
					DateTime.Local.doy = TempDT.UTC.doy
					DateTime.Local.hour = TempDT.UTC.hour
					DateTime.Local.minute = TempDT.UTC.minute
					DateTime.Local.second = TempDT.UTC.second
				endif
						
			 endif
			 
		endelse 
	
	
		thisDateTime(i) = DateTime
		
	endfor
	
	return
end

pro DOY_to_MD, DateTime
  	NoLeapYear = [0,31,28,31,30,31,30,31,31,30,31,30,31]
  	LeapYear   = [0,31,29,31,30,31,30,31,31,30,31,30,31]

	yy = DateTime.UTC.year
	if (yy<1900) then yy = yy + 1900
	if ((((yy mod 4) eq 0) and ((yy mod 400) ne 0)) or (yy eq 2000)) then begin
		YearToUse = LeapYear
	endif else begin
		YearToUse = NoLeapYear
	endelse
	i = 0
	done = 0
	ydoy = DateTime.UTC.doy
	DateTime.UTC.month = 0
	DateTime.UTC.day = 0
	while ((i le 11) and (not done)) do begin
		if (ydoy gt YearToUse(i)) then begin
			ydoy = ydoy - YearToUse(i)
			DateTime.UTC.month = DateTime.UTC.month+1
			DateTime.UTC.day = ydoy
			i = i + 1
		endif else begin
			done = 1
			i = i + 1
		endelse
	endwhile
	
	return

end

pro YMD_to_DOY, DateTime
	YMD_to_DayNum,DateTime,daynum
	DayNum_to_YMDDOY,daynum,DateTime
	return
end

pro YMD_to_DayNum, DateTime,daynum
  	MonthDays = [0,31,59,90,120,151,181,212,243,273,304,334,365]
	
	inyear = long(DateTime.UTC.year)
	if (inyear lt 50) then begin
		inyear = inyear + 2000
	 endif else begin
	 	if (inyear lt 100) then begin
	 		inyear = inyear + 1900
	 	endif
	 endelse
	 daynum = long(0)
	 daynum = ishft(long((inyear-1901)*1461),-2) + MonthDays(DateTime.UTC.month-1) + DateTime.UTC.day + 365
	 
	 if ((((inyear mod 4) eq 0) and (DateTime.UTC.month gt 2) and ((inyear mod 400) ne 0)) or ((inyear eq 2000) and (DateTime.UTC.month gt 2)) ) then begin
	 	daynum = daynum + 1
	 endif
	 
	 return
end

pro DayNum_to_YMDDOY,daynum,DateTime
  	MonthDays = [0,31,59,90,120,151,181,212,243,273,304,334,365]
	lyear = long(double(long(4)*long(daynum))/1461.0)
	ldoy  = long(daynum - long(365+(ishft((lyear-1)*1461,-2))))
	lyear = lyear + 1900
	lld   = ((((lyear mod 4) eq 0) and ((lyear mod 400) ne 0)) or (lyear eq 2000)) and (ldoy gt MonthDays(2))
	;if (lyear eq 2000 and ldoy gt MonthDays(2)) then lld=1
	;if (ldoy gt MonthDays[2]) then lld = 1
	lmonth = 0
	;stop
	while (ldoy gt MonthDays(lmonth) + lld) do lmonth = lmonth + 1
	lday = ldoy - MonthDays(lmonth-1)
	
	if (lmonth gt 2) then lday = lday - lld
	
	DateTime.UTC.year = lyear & DateTime.UTC.month = lmonth & DateTime.UTC.day = lday & DateTime.UTC.doy = ldoy
	
	return
end

pro TAI93_to_UTC,DateTime
	LeapSeconds_TAI93_offset = dblarr(28)
	LeapSeconds_DayNum = lonarr(n_elements(LeapSeconds_TAI93_offset))
	LeapSeconds_TAI93_offset = LeapSeconds_TAI93_offset + [-662774417.000D,  -647049616.000D, -631152015.000D, -599616014.000D, $
	 -568080013.000D, -536544012.000D, -504921611.000D, -473385610.000D, -441849609.000D, -410313608.000D, $
	 -363052807.000D, -331516806.000D, -299980805.000D, -236822404.000D, -157852803.000D, -94694402.000D, $
	 -63158401.000D, -15897600.000D, 15638401.000D, 47174402.000D, 94608003.000D, 141868804.000D, 189302405.000D, 410227206.000D, $
         504921607.0D, 615254408.0D, 709862409.0D, 757382409.0D]
	LeapSeconds_DayNum = LeapSeconds_DayNum + long([26298, 26480, 26664, 27029, 27394, 27759, 28125, 28490, 28855, $
     			      29220, 29767, 30132L, 30497L, 31228L, 32142L, 32873L, 33238L, 33785L, $
     			      34150L, 34515L, 35064L, 35611L, 36160L, 38717L, 39813L, 41090L, 42185L, 42735L])
	xx = where(LeapSeconds_TAI93_offset le DateTime.TAI93)
	xidx = max(xx)
	ref_daynum = long(LeapSeconds_DayNum(xidx))
	sec_since_ref = double(DateTime.TAI93-LeapSeconds_TAI93_offset(xidx))
	days_since_ref = double(sec_since_ref/86400.0D)
	daynums_since_ref = long(double(days_since_ref))
	this_daynum = long(ref_daynum)+long(daynums_since_ref)
	DayNum_to_YMDDOY,this_daynum,DateTime
	
	sec_since_ref = sec_since_ref - double(daynums_since_ref*86400.0)
	hour = fix(double(sec_since_ref)/3600.0D)
	sec_since_ref = sec_since_ref - double(hour)*3600.0D
	minute = fix(double(sec_since_ref)/60.0D)
	sec_since_ref = sec_since_ref - double(minute)*60.0D
	second = sec_since_ref

	DateTime.UTC.hour = hour & DateTime.UTC.minute = minute & DateTime.UTC.second = second
	return
end
	
pro UTC_to_TAI93,DateTime

        LeapSeconds_TAI93_offset = dblarr(28)
	LeapSeconds_DayNum = lonarr(n_elements(LeapSeconds_TAI93_offset))
	LeapSeconds_DayNum = LeapSeconds_DayNum + long([26298, 26480, 26664, 27029, 27394, 27759, 28125, 28490, 28855, $
     			      29220, 29767, 30132L, 30497L, 31228L, 32142L, 32873L, 33238L, 33785L, $
     			      34150L, 34515L, 35064L, 35611L, 36160L, 38717L, 39813L, 41090L, 42185L, 42735L])
	LeapSeconds_TAI93_offset = LeapSeconds_TAI93_offset + [-662774417.000D,  -647049616.000D, -631152015.000D, -599616014.000D, $
	 -568080013.000D, -536544012.000D, -504921611.000D, -473385610.000D, -441849609.000D, -410313608.000D, $
	 -363052807.000D, -331516806.000D, -299980805.000D, -236822404.000D, -157852803.000D, -94694402.000D, $
	 -63158401.000D, -15897600.000D, 15638401.000D, 47174402.000D, 94608003.000D, 141868804.000D, 189302405.000D, 410227206.000D, $
          504921607.0D, 615254408.000D, 709862409.0D, 757382409.0D]

	YMD_to_DayNum, DateTime, this_daynum
	xx = where(LeapSeconds_DayNum le this_daynum)
	xidx = max(xx)
	ref_daynum = LeapSeconds_DayNum(xidx)
	sec_since_ref = (this_daynum-ref_daynum)*86400.0D + (DateTime.UTC.hour*3600.0D) + $
			(DateTime.UTC.minute*60.0D)+(DateTime.UTC.second)
	DateTime.TAI93 = LeapSeconds_TAI93_offset(xidx) + sec_since_ref
	return
end

;pro TAI93_to_UTC,DateTime
;	LeapSeconds_TAI93_offset = dblarr(22)
;	LeapSeconds_DayNum = lonarr(22)
;	LeapSeconds_TAI93_offset = LeapSeconds_TAI93_offset + [-662774417.000D,  -647049616.000D, -631152015.000D, -599616014.000D, $
;	 -568080013.000D, -536544012.000D, -504921611.000D, -473385610.000D, -441849619.000D, -410313608.000D, $
;	 -363052807.000D, -331516806.000D, -299980805.000D, -236822404.000D, -157852803.000D, -94694402.000D, $
;	 -63158401.000D, -15897600.000D, 15638401.000D, 47174402.000D, 94608003.000D, 141868804.000D]
;	LeapSeconds_DayNum = LeapSeconds_DayNum + long([26298, 26480, 26664, 27029, 27394, 27759, 28125, 28490, 28855, $
;     			      29220, 29767, 30132, 30497, 31228, 32142, 32873, 33238, 33785, $
;     			      34150, 34515, 35064, 35611])
;	xx = where(LeapSeconds_TAI93_offset le DateTime.TAI93)
;	xidx = max(xx)
;	ref_daynum = long(LeapSeconds_DayNum(xidx))
;	sec_since_ref = double(DateTime.TAI93-LeapSeconds_TAI93_offset(xidx))
;	days_since_ref = double(sec_since_ref/86400.0D)
;	daynums_since_ref = long(double(days_since_ref))
;	this_daynum = long(ref_daynum)+long(daynums_since_ref)
;	DayNum_to_YMDDOY,this_daynum,DateTime
;	
;	sec_since_ref = sec_since_ref - double(daynums_since_ref*86400.0)
;	hour = fix(double(sec_since_ref)/3600.0D)
;	sec_since_ref = sec_since_ref - double(hour)*3600.0D
;	minute = fix(double(sec_since_ref)/60.0D)
;	sec_since_ref = sec_since_ref - double(minute)*60.0D
;	second = sec_since_ref
;
;	DateTime.UTC.hour = hour & DateTime.UTC.minute = minute & DateTime.UTC.second = second
;	return
;end
	
;pro UTC_to_TAI93,DateTime
;	LeapSeconds_TAI93_offset = dblarr(22)
;	LeapSeconds_DayNum = lonarr(22)
;	LeapSeconds_DayNum = LeapSeconds_DayNum + long([26298, 26480, 26664, 27029, 27394, 27759, 28125, 28490, 28855, $
;     			      29220, 29767, 30132, 30497, 31228, 32142, 32873, 33238, 33785, $
;     			      34150, 34515, 35064, 35611])
;	LeapSeconds_TAI93_offset = LeapSeconds_TAI93_offset + [-662774417.000D,  -647049616.000D, -631152015.000D, -599616014.000D, $
;	 -568080013.000D, -536544012.000D, -504921611.000D, -473385610.000D, -441849619.000D, -410313608.000D, $
;	 -363052807.000D, -331516806.000D, -299980805.000D, -236822404.000D, -157852803.000D, -94694402.000D, $
;	 -63158401.000D, -15897600.000D, 15638401.000D, 47174402.000D, 94608003.000D, 141868804.000D]
;
;	YMD_to_DayNum, DateTime, this_daynum
;	xx = where(LeapSeconds_DayNum le this_daynum)
;	xidx = max(xx)
;	ref_daynum = LeapSeconds_DayNum(xidx)
;	sec_since_ref = (this_daynum-ref_daynum)*86400.0D + (DateTime.UTC.hour*3600.0D) + $
;			(DateTime.UTC.minute*60.0D)+(DateTime.UTC.second)
;	DateTime.TAI93 = LeapSeconds_TAI93_offset(xidx) + sec_since_ref
;	return
;end

;+
;   Full date and time conversion between TAI93, UTC, GPS and Local is available with
;   this package.  For simplicity, a new date/time structure is defined containing each
;   of the above elements.  The function NEW_DATETIME_STRUCTURE() creates instances of
;   this structure.  The procedure CALC_DATETIME determines what date/time formats are
;   inferrable from the fields which have been entered by the user and calculates them.
;
;
;
;   NEW_DATETIME_STRUCTURE
;
;   Calling Sequence
;
;	Result = NEW_DATETIME_STRUCTURE(N)
;
;   Arguments
;
;	N
;	Number of array elements to return.  With n=1 a single new date/time 
;	structure is returned.
;
;   Keywords
;
;	none
;
;   Examples
;
;	Allocate a new date/time structure and store year/month/day in it:
;
;	DT = NEW_DATETIME_STRUCTURE(1)
;	DT.UTC.YEAR  = 1997
; 	DT.UTC.MONTH = 7
; 	DT.UTC.DAY   = 1
;
;	Allocate three date/time structures and store TAI93 times in them:
;
;	DTARR = NEW_DATETIME_STRUCTURE(3)
;	DTARR(*).TAI93 = [47174402.000D, 94608003.000D, 141868804.000D]
;
;   Notes
;
;      The elements available in the returned date/time structure(s) are as follows:
;
;      (aDateTime).TAI93
;		  .GPS
;		  .UTC
;		      .YEAR
;		      .MONTH
;		      .DAY
;		      .DOY
;		      .HOUR
;		      .MINUTE
;		      .SECOND
;		  .LOCAL
;		      .YEAR
;		      .MONTH
;		      .DAY
;		      .DOY
;		      .HOUR
;		      .MINUTE
;		      .SECOND
;
;
;
;   CALC_DATETIME
;
;   Calling Sequence
;
;   	CALC_DATETIME, DATETIME
;
;   Arguments
;
;  	DATETIME
;	A date/time structure or array of structures returned from NEW_DATETIME_STRUCTURE.
;
;   Keywords
;
;	LONGITUDE=longitude
;	An optional longitude or array of longitudes from which to compute local times.
;	If DATETIME is passed as an array and LONGITUDE as a scalar, the single longitude
;	will be used for all calculations.
; 
;   Examples
;
;	Compute the day of year from year/month/day:
;
;	DT = NEW_DATETIME_STRUCTURE(1)
;	DT.UTC.YEAR  = 1997
; 	DT.UTC.MONTH = 7
; 	DT.UTC.DAY   = 1
;	CALC_DATETIME, DT
;	PRINT, DT.UTC.DOY
;
;	Convert TAI93 times to UTC and local date/times.  The times are assumed measured
;	at 84 W, 132 W, and 95 E.
;
;	DTARR = NEW_DATETIME_STRUCTURE(3)
;	DTARR(*).TAI93 = [47174402.000D, 94608003.000D, 141868804.000D]
;	CALC_DATETIME, DT, LONGITUDE=[-84.0,-132.0,95.0]
;	PRINT, DT(*).UTC.YEAR, DT(*).UTC.MONTH, DT(*).UTC.DAY, $
;	       DT(*).UTC.HOUR, DT(*).UTC.MINUTE, DT(*).UTC.SECOND
;	PRINT, ' '
;	PRINT, DT(*).LOCAL.YEAR, DT(*).LOCAL.MONTH, DT(*).LOCAL.DAY, $
;              DT(*).LOCAL.HOUR, DT(*).LOCAL.MINUTE, DT(*).LOCAL.SECOND
;
;   Notes
;
;	UTC time conversions are accurate up to and including the leap second announced
;	on July 1, 1997.  Future leap seconds have not yet been announced.
;
;	"Local" time conversions are not by time zone but calculated from a TAI93 second
;	offset of (longitude/360.0)*86400.  They are intended for use in diurnal cycle and
;	similar calculations.
;
;-
