/* Sample C code to read several 
   orbits and sum them to form a
   mini-"flash rate climatology" */

#include <stdio.h>
#include <math.h>
#include "liblisotd_read_LISOTD.h"

main(int argc, char *argv[])
{
	struct lis_orbit TheData;
	struct anOTDBoundset Bounds;
	FILE   *listfile;
	char   afilename[132];
	float  vtgrid[360][180], flgrid[360][180],
		   flrategrid[360][180];
	long    i,j,k, num_fl, num_vt;
	
    Initialize();
    ResetAllBounds(&Bounds);
  
  	/* Zero out the grids */
  	
  	for (i=0; i<360; i++) {
  		for (j=0; j<180; j++) {
  			vtgrid[i][j] = 0.0;
  			flgrid[i][j] = 0.0;
  			flrategrid[i][j] = 0.0;
  		}
  	}
  	
  	/* We assume the list of HDF file 
  	   names is in an external ASCII file 
  	   called "filelist" */
  	   
  	listfile = fopen("filelist","r");
  	
  	while (fgets(afilename,132,listfile) != NULL) { 
  		
	  	TheData = GetData(afilename,&Bounds);	  
	  		
 		num_fl = TheData.point.point_summary.flash_count;
 		num_vt = TheData.point.point_summary.vt_count; 		
 		
 		for (k=0; k<num_fl; k++) {
 			i = (int)
 			    (TheData.point.lightning.flash[k].location[1]+180.0);
 			j = (int)
 				(TheData.point.lightning.flash[k].location[0]+90.0);
 			flgrid[i][j] = flgrid[i][j] + 1.0;
 		}
 		for (k=0; k<num_vt; k++) {
 			i = (int)(TheData.point.viewtime[k].location[1]+180.0);
 			j = (int)(TheData.point.viewtime[k].location[0]+90.0);
 			vtgrid[i][j] = vtgrid[i][j] + 
 				TheData.point.viewtime[k].effective_obs;
 		}		
 		
		FreeData(&TheData);	
		
	}
	
	fclose(listfile);

	/* The flash rate is just the flash count divided by 
	    the effective viewtime */
	    
  	for (i=0; i<360; i++) {
  		for (j=0; j<180; j++) {
			if (vtgrid[i][j] > 0) 
				flrategrid[i][j] = 
					flgrid[i][j]/vtgrid[i][j];
  		}
  	}

	/* flrategrid now contains the flash rates.  Do with it
	   as you please! */

}
