#include "hdf.h"
#ifndef FAIL
#define FAIL -1
#endif
#ifndef FALSE
#define FALSE 0
#define TRUE 1
#endif

/********************************/
/*	LIS Orbit HDF		*/
/********************************/


struct lis_area {
	float64	TAI93_time;
	float32	delta_time;
	int16	observe_time;
	float32	location[2];
	float32	net_radiance;
	float32	footprint;
	int32	address;
	int32	parent_address;
	int32 	child_address;
	int32	child_count;
	int32	grandchild_count;
	int32	greatgrandchild_count;
	int8	approx_threshold;
	uint8	alert_flag;
	int8	cluster_index;
	int8	density_index;
	int8	noise_index;
#if defined(MAC) || defined(macintosh)	
#else
	float32 oblong_index;
	int32 	grouping_sequence;
#endif
	int8	grouping_status;
	};


#if defined(MAC) || defined(macintosh)	

struct lis_flash {
	float64	TAI93_time;
	float32	delta_time;
	int16	observe_time;
	float32	location[2];
	float32	radiance;
	float32	footprint;
	int32	address;
	int32	parent_address;
	int32 	child_address;
	int32	child_count;
	int32	grandchild_count;
	int8	approx_threshold;
	uint8	alert_flag;
	int8	cluster_index;
	int8	density_index;
	int8	noise_index;
	float32	glint_index;
	int8	grouping_status;
	};

#else 

struct lis_flash {
	float64	TAI93_time;
	float32	delta_time;
	int16	observe_time;
	float32	location[2];
	float32	radiance;
	float32	footprint;
	int32	address;
	int32	parent_address;
	int32 	child_address;
	int32	child_count;
	int32	grandchild_count;
	int8	approx_threshold;
	uint8	alert_flag;
	int8	cluster_index;
	int8	density_index;
	int8	noise_index;
	float32	glint_index;
	float32 oblong_index;
	int32 	grouping_sequence;
	int8	grouping_status;
	};
#endif

struct lis_group {
	float64	TAI93_time;
	int16	observe_time;
	float32	location[2];
	float32	radiance;
	float32	footprint;
	int32	address;
	int32	parent_address;
	int32 	child_address;
	int32	child_count;
	int8	approx_threshold;
	uint8	alert_flag;
	int8	cluster_index;
	int8	density_index;
	int8	noise_index;
	float32	glint_index;
#if defined(MAC) || defined(macintosh)	
#else
	float32 oblong_index;
#endif
	int32   grouping_sequence;
	int8	grouping_status;
	};


struct lis_event {
	float64	TAI93_time;
	int16	observe_time;
	float32	location[2];
	float32	radiance;
	float32	footprint;
	int32	address;
	int32	parent_address;
	int8 	x_pixel;
	int8 	y_pixel;
	int16 	bg_value;
	int16 bg_radiance;
	int8	amplitude;
	uint8	sza_index;
	uint8	glint_index;
	int8	approx_threshold;
	uint8	alert_flag;
	int8	cluster_index;
	int8	density_index;
	int8	noise_index;
	int8	bg_value_flag;
	int32	grouping_sequence;
	};

struct lis_point_summary{
	int32	parent_address;
	int32	event_count;
	int32	event_address;
	int32	group_count;
	int32	group_address;
	int32	flash_count;
	int32	flash_address;
	int32	area_count;
	int32	area_address;
	int32	bg_count;
	int32	bg_address;
	int32	vt_count;
	int32	vt_address;
};

struct lis_viewtime{
	float32	location[2];
	int32	TAI93_start;
	int32	TAI93_end;
	float32	effective_obs;
	uint8	alert_flag;
	int8	approx_threshold;
};

struct	lis_bg_summary{
	float64	TAI93_time;
	int32	address;
	float32	boresight[2];
	float32	corners[8];
};

struct lis_orbit_summary{
	int32	id_number;
	float64	TAI93_start;
	char	UTC_start[28];
	float64	GPS_start;
	float64	TAI93_end;
	float32	start_longitude;
	float32	end_longitude;
	int16	point_data_count;
	int16	point_data_address;
	int32	one_second_count;
	int32	one_second_address;
	int16	summary_image_count;
	int16	summary_image_address;
	uint16	inspection_code;
	uint16	configuration_code;
};

struct lis_one_second{
	float64	TAI93_time;
	uint8	alert_summary;
	uint8	instrument_alert;
	uint8	platform_alert;
	uint8	external_alert;
	uint8	processing_alert;
	float32	position_vector[3];
	float32	velocity_vector[3];
	float32	transform_matrix[9];
	float32	solar_vector[3];
	int32	ephemeris_quality_flag;
	int32	attitude_quality_flag;
	int8	boresight_threshold;
	int8	thresholds[16];
	int8	noise_index;
	int16	event_count[6];
};

/********************************/
/*	background HDF		*/
/********************************/

struct lis_bg_orbit_summary{
	int32	id_number;
	float64	TAI93_start;
	char	UTC_start[28];
	float64	GPS_start;
	float64	TAI93_end;
	float32	start_longitude;
	float32	end_longitude;
	int16	image_summary_count;
	int16	image_summary_address;
	int16	image_info_count;
	int16	image_info_address;
};

struct	lis_bg_info{
	float64	TAI93_time;
	uint8	alert_summary;
	uint8	instrument_alert;
	uint8	platform_alert;
	uint8	external_alert;
	uint8	processing_alert;
	float32	position_vector[3];
	float32	velocity_vector[3];
	float32	transform_matrix[9];
	float32	solar_vector[3];
	int32	ephemeris_quality_flag;
	int32	attitude_quality_flag;
	int16	bg_value;
	int8	noise_index;
	int16	event_count[6];
};

struct lis_bg_data_summary{
	float64	TAI93_time;
	int32	address;
	float32	boresight[2];
	float32	corners[8];
};




/********************/
/* LIS read protos  */
/********************/
long OPEN_lis_orbit_hdf(char *hdf_fname);
long OPEN_lis_background_hdf(char *hdf_fname);

long CLOSE_lis_orbit_hdf(long file_number);
long CLOSE_lis_background_hdf(long file_number);


 

long READ_lis_area(long file_number,struct lis_area *lis_a,long record);
long READ_lis_flash(long file_number,struct lis_flash *lis_f,long record);
long READ_lis_group(long file_number,struct lis_group *lis_g,long record);
long READ_lis_event(long file_number,struct lis_event *lis_e,long record);
long READ_lis_point_summary(long file_number,struct lis_point_summary *lis_ps,long record);
long READ_lis_viewtime(long file_number,struct lis_viewtime *lis_vt,long record);
long READ_lis_bg_summary(long file_number,struct lis_bg_summary *lis_bs,long record);
long READ_lis_orbit_summary(long file_number,struct lis_orbit_summary *lis_os,long record);
long READ_lis_one_second(long file_number,struct lis_one_second *lis_ones,long record);

long READ_lis_areas(long file_number,struct lis_area *lis_a,long record,long numrec);
long READ_lis_flashes(long file_number,struct lis_flash *lis_f,long record,long numrec);
long READ_lis_groups(long file_number,struct lis_group *lis_g,long record,long numrec);
long READ_lis_events(long file_number,struct lis_event *lis_e,long record,long numrec);
long READ_lis_viewtimes(long file_number,struct lis_viewtime *lis_vt,long record,long numrec);
long READ_lis_bg_summaries(long file_number,struct lis_bg_summary *lis_bs,long record,long numrec);
long READ_lis_one_seconds(long file_number,struct lis_one_second *lis_ones,long record,long numrec);

long lis_area_COUNT(long file_number);
long lis_flash_COUNT(long file_number);
long lis_group_COUNT(long file_number);
long lis_event_COUNT(long file_number);
long lis_point_summary_COUNT(long file_number);
long lis_viewtime_COUNT(long file_number);
long lis_bg_summary_COUNT(long file_number);
long lis_orbit_summary_COUNT(long file_number);
long lis_one_second_COUNT(long file_number);


long READ_lis_bg_orbit_summary(long file_number,struct lis_bg_orbit_summary *lis_bos,long record);
long READ_lis_bg_info(long file_number,struct lis_bg_info *lis_bg,long record);
long READ_lis_bg_data_summary(long file_number,struct lis_bg_data_summary *lis_bsd,long record);
long READ_lis_background_data(long file_number,short int *lis_image,long *vsrecord);

long lis_bg_orbit_summary(long file_number);
long lis_bg_info(long file_number);
long lis_bg_data_summary(long file_number);
long lis_background_data(long file_number);




struct otd_AREA {
	long		seq;
	double		TAI93;		
	float		delta;
	float		view;
	short int 	s_z_a;
	unsigned char	d_n_t;
	unsigned char	end_status;
	short int	events;
	float		cent[2];
	float		stdev[2];
	short int 	loc_count;
	float		rad;
	long		orbit_id;
	long		day;
	long		children;
	long		child_seq;
	long		child_rec;
	short int	QA[4];
	};

struct otd_FLASH {
	long		seq;
	double		TAI93;		
	float		delta;
	float		view;
	short int 	s_z_a;
	unsigned char	d_n_t;
	unsigned char	end_status;
	short int	events;
	float		cent[2];
	float		stdev[2];
	short int 	loc_count;
	float		rad;
	long		parent_seq;
	long		parent_rec;
	long		children;
	long		child_seq;
	long		child_rec;
	short int	QA[4];
	};

struct otd_GROUP {
	long		seq;
	double		TAI93;		
	float		delta;
	float		view;
	short int 	s_z_a;
	unsigned char	d_n_t;
	unsigned char	end_status;
	short int	events;
	float		cent[2];
	float		stdev[2];
	short int 	loc_count;
	float		rad;
	long		parent_seq;
	long		parent_rec;
	long		children;
	long		child_seq;
	long		child_rec;
	short int	QA[4];
	};

struct otd_EVENT {
	long		event_number;
	long		seq_number;
	double		TAI93;
	short int	s_z_a;
	unsigned char	d_n_t;
	short int	x_pixel;
	short int	y_pixel;
	short int	raw_radiance;
	float		cal_radiance;
	long		parent_seq;
	long		parent_rec;
	short int	QA[4];
	float		location[2];
	};
	
struct otd_BROWSE {
	float		location[2];
	short int	EGFA[4];
	float		EGFA_cal_rad[4];
	float		max_rate;
	short int	single_event_areas;
	short int	highres_locs;
	long		first_highres;
	long		bkgnds;
	short int	iloper;
	long		bkgnd_pointer;
	short int	duration;
	};

struct otd_VECTOR {
	float		location[2];
	short int	EGFA_counts[4];
	float		EGFA_cal_radiance[4];
	float		max_flash_rate;
	short int	single_event_areas;
	};

struct otd_IMAGE_ATTRIBUTE {
	double		TAI93;
	float		NW_location[2];
	float		NE_location[2];
	float		SW_location[2];
	float		SE_location[2];
	float		nadir_location[2];
	long		image_pointer;
	long		calibration_number;
	long		spice_pointer;
	};

struct otd_THRESHOLD {
	double		TAI93;
	unsigned char	values[16];
	};

struct otd_DENSITY {
	float		location[2];
	long		flash_count;
	};
struct otd_Orbit_attributes {
	long		ID;
	double		TAI93_start;
	float		start_location[2];
	double		TAI93_end;
	float		end_location[2];
	};

struct otd_View_time {
	float		grid_point[2];
	double		TAI93_start;
	short int	duration;
	};
struct otd_EPHEMERIS {
	double		GPS;
	double		kernel[15];
	};

long close_OTD_HDF_FILE();



/************************/
/* LIS/OTD read protos  */
/************************/
long open_OTD_HDF_READONLY(char *hdf_fname);


 
long read_otd_IMAGE(short int *image,long *vsrecord);

long read_otd_EVENT(void *databuffer,long *otd_event_vrecord);	
long read_otd_GROUP(void *databuffer,long *otd_group_vrecord); 
long read_otd_FLASH(void *databuffer,long *otd_flash_vrecord);
long read_otd_AREA(void *databuffer,long *otd_area_vrecord);	
long read_otd_VECTOR_PRODUCTS(void *databuffer,long *otd_vector_vrecord); 
long read_otd_IMAGE_ATTRIBUTE(void *databuffer,long *otd_image_vrecord);  
long read_otd_ORBIT_DATA(void *databuffer,long *otd_image_vrecord);	
long read_otd_METADATA(void *databuffer,long *otd_metadata_vrecord);	
long read_otd_INVENTORY_METADATA(void *databuffer,long *otd_inventory_metadata_vrecord);	
long read_otd_ARCHIVED_METADATA(void *databuffer,long *otd_archived_metadata_vrecord);	
long read_otd_BROWSE_DATA(void *databuffer,long *otd_browse_data_vrecord); 

long read_otd_ORBIT_SUMMARY(void *databuffer,long *otd_orbit_summary_vrecord); 
long read_otd_ORBIT_FLASH_RATE(void *databuffer,long *otd_orbit_flash_rate_vrecord);
long read_otd_ORBIT_EVENT_RATE(void *databuffer,long *otd_orbit_event_rate_vrecord);
long read_otd_EPHEMERIS(void *databuffer,long *otd_ephemeris_vrecord); 
long read_otd_FLASH_RATE(void *databuffer,long *otd_orbit_flash_rate_vrecord); 
long read_otd_EVENT_RATE(void *databuffer,long *otd_orbit_event_rate_vrecord); 

long read_otd_VIEW_TIME(void *databuffer,long *otd_view_vrecord);
long read_otd_GRID_25(void *databuffer,long *otd_grid_25_vrecord);
long read_otd_GRID_50(void *databuffer,long *otd_grid_50_vrecord);


long read_otd_area_struct(struct otd_AREA *otd_area,long record);
long read_otd_flash_struct(struct otd_FLASH *otd_flash,long record);
long read_otd_group_struct(struct otd_GROUP *otd_group,long record);
long read_otd_event_struct(struct otd_EVENT *otd_event,long record);
long read_otd_threshold_struct(struct otd_THRESHOLD *otd_threshold,long record);
long read_otd_browse_struct(struct otd_BROWSE *otd_browse,long record);
long read_otd_image_attribute_struct(struct otd_IMAGE_ATTRIBUTE *otd_image,long record);
long read_otd_25_grid_struct(struct otd_DENSITY *otd_grid_25,long record);
long read_otd_500_km_grid_struct(struct otd_DENSITY *otd_grid_500_km,long record);
long read_otd_view_time_struct(struct otd_View_time *otd_view_time,long record);
long read_otd_ephemeris_struct(struct otd_EPHEMERIS *otd_eph,long record);
long read_otd_orbit_attributes_struct(struct otd_Orbit_attributes *otd_oa,long record);

long READ_otd_ephemerises(struct otd_EPHEMERIS *otd_ephemeris_var, long record, long numrec);
long READ_otd_viewtimes(struct otd_View_time *otd_viewtime_var, long record, long numrec);
long READ_otd_events(struct otd_EVENT *otd_event_var, long record, long numrec);
long READ_otd_groups(struct otd_GROUP *otd_group_var, long record, long numrec);
long READ_otd_flashes(struct otd_FLASH *otd_flash_var, long record, long numrec);
long READ_otd_areas(struct otd_AREA *otd_area_var, long record, long numrec);


long get_total_otd_areas();
long get_total_otd_flashes();
long get_total_otd_groups();
long get_total_otd_events();

long get_total_otd_metadata();
long get_total_otd_threshold();
long get_total_otd_browse();
long get_total_otd_image();
long get_total_otd_25_grid();
long get_total_otd_500_km();
long get_total_otd_view_time();
long get_total_otd_ephemeris();
long get_total_otd_flash_rate();
long get_total_otd_event_rate();






