#include "liblisotd_read_LISOTD.h"
#include "liblisotd_err_defs.h"

struct lis_orbit GetData(hdfname, Bounds)
char hdfname[132];
struct anOTDBoundset *Bounds;
{
	struct lis_orbit orbit;
	struct anOTDDataset otd_orbit;
	
	/* Test for platform */
	if (strstr(hdfname, "mlab")==NULL) {
		if (strstr(hdfname,"LIS_SC")==NULL) {
			fprintf(stderr,"Unknown file format - name must include either 'mlab' or 'LIS_SC'\n");
			exit(-1);
		} else {
			sprintf(orbit.platform,"LIS");
			orbit = GetLISData(hdfname,Bounds);
		}
	} else {
		sprintf(orbit.platform,"OTD");
		otd_orbit = GetOTDData(hdfname,Bounds);
		orbit = OTD_to_LIS(&otd_orbit);
	}
	return(orbit);
}

struct lis_orbit GetLISData(hdfname, Bounds)
char hdfname[132];
struct anOTDBoundset	*Bounds;
{
	long fid, num_a, num_f, num_g, num_e, num_vt, num_bg, num_os, ret;
	struct lis_orbit	orbit;
	
	/* Open the file */
	
    fid = OPEN_lis_orbit_hdf(hdfname);
    if((fid<0)&&((fid&0xffffff00)!=lis_NoMetadata)){
            printf("OPEN_lis_orbit_hdf failed gflash %s : %08x\n",hdfname,fid);
            exit(-1);
            fid = fid&0xff;
    }
    if(((fid&0xffffff00)==lis_NoMetadata))fid = fid^lis_NoMetadata;

	/* Count the vdatas */
	
	num_a = lis_area_COUNT(fid);
	num_f = lis_flash_COUNT(fid);
	num_g = lis_group_COUNT(fid);
	num_e = lis_event_COUNT(fid);
	num_vt = lis_viewtime_COUNT(fid);
	num_bg = lis_bg_summary_COUNT(fid);
	num_os = lis_one_second_COUNT(fid);

	/* Allocate the subarrays */
	
	orbit.point.lightning.area  	= (struct lis_area *)calloc(num_a,sizeof(struct lis_area));
	orbit.point.lightning.flash 	= (struct lis_flash *)calloc(num_f,sizeof(struct lis_flash));
	orbit.point.lightning.group		= (struct lis_group *)calloc(num_g,sizeof(struct lis_group));
	orbit.point.lightning.event		= (struct lis_event *)calloc(num_e,sizeof(struct lis_event));
	orbit.point.viewtime 			= (struct lis_viewtime *)calloc(num_vt,sizeof(struct lis_viewtime));
	orbit.point.bg_summary 			= (struct lis_bg_summary *)calloc(num_bg,sizeof(struct lis_bg_summary));
	orbit.one_second				= (struct lis_one_second *)calloc(num_os,sizeof(struct lis_one_second));
	
	/* Read the data */
	
	if (num_a > 0) {
		ret = READ_lis_areas(fid, orbit.point.lightning.area, 0, num_a);
	}
	if (num_f > 0) {
		ret = READ_lis_flashes(fid, orbit.point.lightning.flash, 0, num_f);
	}
	if (num_g > 0) {
		ret = READ_lis_groups(fid, orbit.point.lightning.group, 0, num_g);
	}
	if (num_e > 0) {
		ret = READ_lis_events(fid, orbit.point.lightning.event, 0, num_e);
	}
	if (num_vt > 0) {
		ret = READ_lis_viewtimes(fid, orbit.point.viewtime, 0, num_vt);
	}
	if (num_bg > 0) {
		ret = READ_lis_bg_summaries(fid, orbit.point.bg_summary, 0, num_bg);
	}
	if (num_os > 0) {
		ret = READ_lis_one_seconds(fid, orbit.one_second, 0, num_os);
	}
  	ret = READ_lis_orbit_summary(fid,&orbit.orbit_summary, 0);
  	ret = READ_lis_point_summary(fid,&orbit.point.point_summary, 0);
	
    if(CLOSE_lis_orbit_hdf(fid)!=lis_operation_SUCCESSFUL )printf("close %s file failed\n",hdfname);

	return(orbit);
}

struct lis_orbit	OTD_to_LIS(struct anOTDDataset *OTD_Data) 
{
	/* Note ... the following fields are not yet populated...
		LIS_Data.one_second[i].solar_vector[0] 
		LIS_Data.one_second[i].solar_vector[1] 
		LIS_Data.one_second[i].solar_vector[2] 
		LIS_Data.point.lightning.group[i].oblong_index	
		LIS_Data.point.lightning.flash[i].oblong_index	
		LIS_Data.point.lightning.area[i].footprint		
		LIS_Data.point.lightning.area[i].oblong_index	
	*/	
	struct lis_orbit		LIS_Data;
	struct aUTCDateTime		UTC;
	long   i, j, k, ephem_idx;
	double thr_TAI[7];
	short int thr_val[7], num_thr, this_thr=63;
	float pixres;
	
	num_thr=7;
	
	thr_TAI[0] = 0.0;
	thr_TAI[1] = 70848002.0;
	thr_TAI[2] = 76876442.0;
	thr_TAI[3] = 77155202.0;
	thr_TAI[4] = 80392502.0;
	thr_TAI[5] = 120199443.0;
	thr_TAI[6] = 999999999.0;
	
	thr_val[0] = 63;
	thr_val[1] = 14;
	thr_val[2] = 20;
	thr_val[3] = 18;
	thr_val[4] = 17;
	thr_val[5] = 15;
	thr_val[6] = 63;
	
	/* Orbit summary */
	
	LIS_Data.orbit_summary.id_number 	= OTD_Data->Orbits[0].Attributes.ID;
	LIS_Data.orbit_summary.TAI93_start 	= OTD_Data->Orbits[0].Attributes.TAI93_start;
	UTC = TAI93_to_UTC(LIS_Data.orbit_summary.TAI93_start);
	sprintf(LIS_Data.orbit_summary.UTC_start,"%4d-%02d-%02dT%02d:%02d:%09.06fZ", 
		UTC.year,UTC.month,UTC.day,UTC.hour,UTC.minute,UTC.second  );
	LIS_Data.orbit_summary.GPS_start 	= UTC_to_GPS(UTC);
	LIS_Data.orbit_summary.TAI93_end	= OTD_Data->Orbits[0].Attributes.TAI93_end;
	LIS_Data.orbit_summary.start_longitude	= OTD_Data->Orbits[0].Attributes.start_location[1];
	LIS_Data.orbit_summary.end_longitude	= OTD_Data->Orbits[0].Attributes.end_location[1];
	LIS_Data.orbit_summary.point_data_count		= 1;
	LIS_Data.orbit_summary.point_data_address 	= 0;
	LIS_Data.orbit_summary.one_second_count		= 6100;
	LIS_Data.orbit_summary.one_second_address	= 0;
	LIS_Data.orbit_summary.summary_image_count	= 0;
	LIS_Data.orbit_summary.summary_image_address= 0;
	LIS_Data.orbit_summary.inspection_code		= 0;
	LIS_Data.orbit_summary.configuration_code	= 0;
	
	for (i=0; i < num_thr-1; i++) {
		if ((LIS_Data.orbit_summary.TAI93_start >= thr_TAI[i]) &&
			(LIS_Data.orbit_summary.TAI93_start < thr_TAI[i+1])) {
				this_thr = thr_val[i];
		}
	}
	
	/* Point summary */
	
	LIS_Data.point.point_summary.parent_address = 	0;
	LIS_Data.point.point_summary.event_count	=	OTD_Data->num_events;
	LIS_Data.point.point_summary.event_address	=	0;
	LIS_Data.point.point_summary.group_count	=	OTD_Data->num_groups;
	LIS_Data.point.point_summary.group_address 	=	0;
	LIS_Data.point.point_summary.flash_count	=	OTD_Data->num_flashes;
	LIS_Data.point.point_summary.flash_address	=	0;
	LIS_Data.point.point_summary.area_count		=	OTD_Data->num_areas;
	LIS_Data.point.point_summary.area_address	=	0;
	LIS_Data.point.point_summary.bg_count		=	OTD_Data->Orbits[0].num_backgrounds;
	LIS_Data.point.point_summary.bg_address		=	0;
	LIS_Data.point.point_summary.vt_count		=	OTD_Data->Orbits[0].num_viewtimes;
	LIS_Data.point.point_summary.vt_address		=	0;
	
	/* Allocate memory */
	
	LIS_Data.point.lightning.area  		= (struct lis_area 	*)calloc(LIS_Data.point.point_summary.area_count,sizeof(struct lis_area));
	LIS_Data.point.lightning.flash 		= (struct lis_flash *)calloc(LIS_Data.point.point_summary.flash_count,sizeof(struct lis_flash));
	LIS_Data.point.lightning.group		= (struct lis_group *)calloc(LIS_Data.point.point_summary.group_count,sizeof(struct lis_group));
	LIS_Data.point.lightning.event		= (struct lis_event *)calloc(LIS_Data.point.point_summary.event_count,sizeof(struct lis_event));
	LIS_Data.point.viewtime 			= (struct lis_viewtime *)calloc(LIS_Data.point.point_summary.vt_count,sizeof(struct lis_viewtime));
	LIS_Data.point.bg_summary 			= (struct lis_bg_summary *)calloc(LIS_Data.point.point_summary.bg_count,sizeof(struct lis_bg_summary));
	LIS_Data.one_second					= (struct lis_one_second *)calloc(6100,sizeof(struct lis_one_second));

	/* One Seconds */	
	
	for (i=0; i<6000; i++) {
		LIS_Data.one_second[i].TAI93_time 		= (long)(OTD_Data->Orbits[0].Attributes.TAI93_start + i);
		ephem_idx = (long)(LIS_Data.one_second[i].TAI93_time - OTD_Data->Orbits[0].Ephemeris[0].GPS);
		if (ephem_idx >= 0 && ephem_idx < OTD_Data->Orbits[0].num_ephemeris) {
			LIS_Data.one_second[i].position_vector[0] = OTD_Data->Orbits[0].Ephemeris[ephem_idx].kernel[0];
			LIS_Data.one_second[i].position_vector[1] = OTD_Data->Orbits[0].Ephemeris[ephem_idx].kernel[1];
			LIS_Data.one_second[i].position_vector[2] = OTD_Data->Orbits[0].Ephemeris[ephem_idx].kernel[2];
			LIS_Data.one_second[i].velocity_vector[0] = OTD_Data->Orbits[0].Ephemeris[ephem_idx].kernel[3];
			LIS_Data.one_second[i].velocity_vector[1] = OTD_Data->Orbits[0].Ephemeris[ephem_idx].kernel[4];
			LIS_Data.one_second[i].velocity_vector[2] = OTD_Data->Orbits[0].Ephemeris[ephem_idx].kernel[5];
			LIS_Data.one_second[i].transform_matrix[0] = OTD_Data->Orbits[0].Ephemeris[ephem_idx].kernel[6];
			LIS_Data.one_second[i].transform_matrix[1] = OTD_Data->Orbits[0].Ephemeris[ephem_idx].kernel[7];
			LIS_Data.one_second[i].transform_matrix[2] = OTD_Data->Orbits[0].Ephemeris[ephem_idx].kernel[8];
			LIS_Data.one_second[i].transform_matrix[3] = OTD_Data->Orbits[0].Ephemeris[ephem_idx].kernel[9];
			LIS_Data.one_second[i].transform_matrix[4] = OTD_Data->Orbits[0].Ephemeris[ephem_idx].kernel[10];
			LIS_Data.one_second[i].transform_matrix[5] = OTD_Data->Orbits[0].Ephemeris[ephem_idx].kernel[11];
			LIS_Data.one_second[i].transform_matrix[6] = OTD_Data->Orbits[0].Ephemeris[ephem_idx].kernel[12];
			LIS_Data.one_second[i].transform_matrix[7] = OTD_Data->Orbits[0].Ephemeris[ephem_idx].kernel[13];
			LIS_Data.one_second[i].transform_matrix[8] = OTD_Data->Orbits[0].Ephemeris[ephem_idx].kernel[14];
		}
		LIS_Data.one_second[i].boresight_threshold    = this_thr;
		for (j=0; j<16; j++) {
			LIS_Data.one_second[i].thresholds[j] = this_thr;
		}
		/*
		LIS_Data.one_second[i].solar_vector[0] =
		LIS_Data.one_second[i].solar_vector[1] =
		LIS_Data.one_second[i].solar_vector[2] =
		*/
		/* FIFO Full */
		if (OTD_Data->Orbits[0].Platform_Data.FIFO_full.Datum[i] > 0) {
			LIS_Data.one_second[i].instrument_alert += 4;
		}
		/* Out of control */
		if (OTD_Data->Orbits[0].Platform_Data.Out_of_control.Datum[i] > 5) {
			LIS_Data.one_second[i].platform_alert += 4+16;
			LIS_Data.one_second[i].processing_alert += 64;
		    LIS_Data.one_second[i].ephemeris_quality_flag += 1 + 2 + 4 + 32 + 2048 + 4096 + 8192;
		    LIS_Data.one_second[i].attitude_quality_flag += 1 + 2 + 4 + 32 + 2048 + 4096 + 8192;
		}
		/* No ephemeris */
		if (OTD_Data->Orbits[0].Platform_Data.No_ephemeris.Datum[i] > 15 ||
		    OTD_Data->Orbits[0].Platform_Data.No_ephemeris.Datum[i] > 15) {
		    LIS_Data.one_second[i].platform_alert += 2 + 8;
		    LIS_Data.one_second[i].processing_alert += 64;
		    LIS_Data.one_second[i].ephemeris_quality_flag += 1 + 2 + 64 + 128;
		    LIS_Data.one_second[i].attitude_quality_flag += 1 + 2 + 64 + 128;
		}
		/* Warmup */
		if (OTD_Data->Orbits[0].Platform_Data.Warmup.Datum[i] > 15) {
			LIS_Data.one_second[i].instrument_alert += 16;
		}
		/* Connector */
		if (OTD_Data->Orbits[0].Platform_Data.Connector_problem.Datum[i] > 30) {
			LIS_Data.one_second[i].instrument_alert += 16+32;
		}
		/* Alert summary */
			/* Instrument fatal */
			k = ((LIS_Data.one_second[i].instrument_alert>>0)&1) ||
				((LIS_Data.one_second[i].instrument_alert>>2)&1) ||
				((LIS_Data.one_second[i].instrument_alert>>4)&1) ||
				((LIS_Data.one_second[i].instrument_alert>>6)&1);
			LIS_Data.one_second[i].alert_summary += k*1;
			/* Instrument warning */
			k = ((LIS_Data.one_second[i].instrument_alert>>2)&1) ||
				((LIS_Data.one_second[i].instrument_alert>>3)&1) ||
				((LIS_Data.one_second[i].instrument_alert>>5)&1) ||
				((LIS_Data.one_second[i].instrument_alert>>7)&1);
			LIS_Data.one_second[i].alert_summary += k*2;
			/* Platform fatal */
			k = ((LIS_Data.one_second[i].platform_alert>>1)&1) ||
				((LIS_Data.one_second[i].platform_alert>>3)&1) ||
				((LIS_Data.one_second[i].platform_alert>>5)&1);
			LIS_Data.one_second[i].alert_summary += k*4;
			/* Platform warning */
			k = ((LIS_Data.one_second[i].platform_alert>>0)&1) ||
				((LIS_Data.one_second[i].platform_alert>>2)&1) ||
				((LIS_Data.one_second[i].platform_alert>>4)&1) ||
				((LIS_Data.one_second[i].platform_alert>>6)&1);
			LIS_Data.one_second[i].alert_summary += k*8;
			/* No external flags cause fatal condition */
			/* External warning */
			k = ((LIS_Data.one_second[i].external_alert>>0)&1) ||
				((LIS_Data.one_second[i].external_alert>>1)&1) ||
				((LIS_Data.one_second[i].external_alert>>2)&1);
			LIS_Data.one_second[i].alert_summary += k*32;
			/* Processing fatal */
			k = ((LIS_Data.one_second[i].processing_alert>>1)&1) ||
				((LIS_Data.one_second[i].processing_alert>>2)&1) ||
				((LIS_Data.one_second[i].processing_alert>>3)&1) ||
				((LIS_Data.one_second[i].processing_alert>>4)&1);
			LIS_Data.one_second[i].alert_summary += k*64;
			/* Processing warning */
			k = ((LIS_Data.one_second[i].processing_alert>>0)&1) ||
				((LIS_Data.one_second[i].processing_alert>>4)&1) ||
				((LIS_Data.one_second[i].processing_alert>>5)&1) ||
				((LIS_Data.one_second[i].processing_alert>>6)&1);
			LIS_Data.one_second[i].alert_summary += k*128;

#if defined(MAC) || defined(macintosh)	
#else
		LIS_Data.one_second[i].event_count[0]	 = OTD_Data->Orbits[0].Platform_Data.Rate_Prefilter.Datum[i];
		LIS_Data.one_second[i].event_count[1]	 = OTD_Data->Orbits[0].Platform_Data.Rate_Postglint.Datum[i];
		LIS_Data.one_second[i].event_count[2]	 = OTD_Data->Orbits[0].Platform_Data.Rate_Postnoise.Datum[i];
		LIS_Data.one_second[i].event_count[3]	 = OTD_Data->Orbits[0].Platform_Data.Rate_Postnoise.Datum[i];
		LIS_Data.one_second[i].event_count[4]	 = OTD_Data->Orbits[0].Platform_Data.Rate_Postnoise.Datum[i];
		LIS_Data.one_second[i].event_count[5]	 = OTD_Data->Orbits[0].Platform_Data.Rate_Postnoise.Datum[i];
		if (LIS_Data.one_second[i].event_count[0] == 0) {
			LIS_Data.one_second[i].noise_index = 0;
			LIS_Data.one_second[i].instrument_alert = LIS_Data.one_second[i].instrument_alert + 1;
		} else {
			LIS_Data.one_second[i].noise_index = (int)(100.0*
				((float)LIS_Data.one_second[i].event_count[5]/(float)LIS_Data.one_second[i].event_count[0]));
		}
#endif
	}
	
	/* Events */
	
	for (i=0; i<LIS_Data.point.point_summary.event_count; i++) {
		LIS_Data.point.lightning.event[i].TAI93_time 	= OTD_Data->Events[i].HDFEvent.TAI93;
		LIS_Data.point.lightning.event[i].location[0]	= OTD_Data->Events[i].HDFEvent.location[0];
		LIS_Data.point.lightning.event[i].location[1]	= OTD_Data->Events[i].HDFEvent.location[1];
		LIS_Data.point.lightning.event[i].radiance		= OTD_Data->Events[i].radiance;
		LIS_Data.point.lightning.event[i].address		= i;
		LIS_Data.point.lightning.event[i].parent_address= OTD_Data->Events[i].HDFEvent.parent_rec;
		LIS_Data.point.lightning.event[i].x_pixel		= OTD_Data->Events[i].HDFEvent.x_pixel;
		LIS_Data.point.lightning.event[i].y_pixel		= OTD_Data->Events[i].HDFEvent.y_pixel;
		LIS_Data.point.lightning.event[i].bg_value		=
		LIS_Data.point.lightning.event[i].bg_radiance	=
		LIS_Data.point.lightning.event[i].amplitude		= OTD_Data->Events[i].HDFEvent.raw_radiance;
		LIS_Data.point.lightning.event[i].sza_index		= OTD_Data->Events[i].HDFEvent.s_z_a;
		LIS_Data.point.lightning.event[i].approx_threshold = this_thr;
		LIS_Data.point.lightning.event[i].cluster_index	= (unsigned char)OTD_Data->Events[i].HDFEvent.QA[0];
		LIS_Data.point.lightning.event[i].noise_index	= (unsigned char)(OTD_Data->Events[i].HDFEvent.QA[2]/10);
		LIS_Data.point.lightning.event[i].glint_index	= (unsigned char)(OTD_Data->Events[i].HDFEvent.QA[1]);
		k = OTD_Data->Events[i].HDFEvent.QA[3];
		if (k > 255) k=255;
		LIS_Data.point.lightning.event[i].density_index	= k;
		k = (long)(LIS_Data.point.lightning.event[i].TAI93_time-LIS_Data.one_second[0].TAI93_time);
		if ((k>=0) && (k<6100)) {
			LIS_Data.point.lightning.event[i].alert_flag = LIS_Data.one_second[k].alert_summary;
		}
		pixres = PixelResolutionKm( LIS_Data.point.lightning.event[i].x_pixel,
							  		LIS_Data.point.lightning.event[i].y_pixel,"OTD");
		LIS_Data.point.lightning.event[i].footprint = pixres*pixres;
			
		LIS_Data.point.lightning.event[i].bg_value_flag = 0;
		LIS_Data.point.lightning.event[i].grouping_sequence = OTD_Data->Events[i].HDFEvent.seq_number;
		LIS_Data.point.lightning.event[i].observe_time	= 
			OTD_Data->Groups[LIS_Data.point.lightning.event[i].parent_address].HDFGroup.view;
	}

	/* Groups */
	
	for (i=0; i<LIS_Data.point.point_summary.group_count; i++) {
		LIS_Data.point.lightning.group[i].TAI93_time 	= OTD_Data->Groups[i].HDFGroup.TAI93;
		LIS_Data.point.lightning.group[i].observe_time	= OTD_Data->Groups[i].HDFGroup.view;
		LIS_Data.point.lightning.group[i].location[0]	= OTD_Data->Groups[i].HDFGroup.cent[0];
		LIS_Data.point.lightning.group[i].location[1]	= OTD_Data->Groups[i].HDFGroup.cent[1];
		LIS_Data.point.lightning.group[i].radiance		= OTD_Data->Groups[i].radiance;
		LIS_Data.point.lightning.group[i].address		= i;
		LIS_Data.point.lightning.group[i].parent_address= OTD_Data->Groups[i].Parent;
		LIS_Data.point.lightning.group[i].child_address	= OTD_Data->Groups[i].FirstEvent;
		LIS_Data.point.lightning.group[i].child_count	= OTD_Data->Groups[i].Events_Accepted;
		LIS_Data.point.lightning.group[i].approx_threshold = this_thr;
		LIS_Data.point.lightning.group[i].cluster_index = (unsigned char)OTD_Data->Groups[i].HDFGroup.QA[0];
		LIS_Data.point.lightning.group[i].noise_index	= (unsigned char)(OTD_Data->Groups[i].HDFGroup.QA[2]/10);
		LIS_Data.point.lightning.group[i].glint_index	= (unsigned char)(OTD_Data->Groups[i].HDFGroup.QA[1]);
		k = OTD_Data->Groups[i].HDFGroup.QA[3];
		if (k > 255) k=255;
		LIS_Data.point.lightning.group[i].density_index	= k;
		k = (long)(LIS_Data.point.lightning.group[i].TAI93_time-LIS_Data.one_second[0].TAI93_time);
		if ((k>=0) && (k<6100)) {
			LIS_Data.point.lightning.group[i].alert_flag = LIS_Data.one_second[k].alert_summary;
		}
		for (j=LIS_Data.point.lightning.group[i].child_address; 
		     j<LIS_Data.point.lightning.group[i].child_address+LIS_Data.point.lightning.group[i].child_count;
		     j++) {
		     LIS_Data.point.lightning.group[i].footprint += LIS_Data.point.lightning.event[j].footprint;
		}
		/* These must be populated specially, some later... 
		LIS_Data.point.lightning.group[i].oblong_index	=
		*/
#if defined(MAC) || defined(macintosh)	
#else
		LIS_Data.point.lightning.group[i].grouping_sequence = OTD_Data->Groups[i].HDFGroup.seq;
#endif
		LIS_Data.point.lightning.group[i].grouping_status = OTD_Data->Groups[i].HDFGroup.end_status;
	}
	
	/* Flashes */
	
	for (i=0; i<LIS_Data.point.point_summary.flash_count; i++) {
		LIS_Data.point.lightning.flash[i].TAI93_time 	= OTD_Data->Flashes[i].HDFFlash.TAI93;
		LIS_Data.point.lightning.flash[i].delta_time 	= OTD_Data->Flashes[i].HDFFlash.delta;
		LIS_Data.point.lightning.flash[i].observe_time 	= OTD_Data->Flashes[i].HDFFlash.view;
		LIS_Data.point.lightning.flash[i].location[0] 	= OTD_Data->Flashes[i].HDFFlash.cent[0];
		LIS_Data.point.lightning.flash[i].location[1]	= OTD_Data->Flashes[i].HDFFlash.cent[1];
		LIS_Data.point.lightning.flash[i].radiance		= OTD_Data->Flashes[i].radiance;
		LIS_Data.point.lightning.flash[i].footprint		= OTD_Data->Flashes[i].size;
		LIS_Data.point.lightning.flash[i].address		= i;
		LIS_Data.point.lightning.flash[i].parent_address= OTD_Data->Flashes[i].Parent;
		LIS_Data.point.lightning.flash[i].child_address = OTD_Data->Flashes[i].FirstGroup;
		LIS_Data.point.lightning.flash[i].child_count	= OTD_Data->Flashes[i].Groups_Accepted;
		LIS_Data.point.lightning.flash[i].approx_threshold	= this_thr;
		LIS_Data.point.lightning.flash[i].cluster_index = (unsigned char)OTD_Data->Flashes[i].HDFFlash.QA[0];
		LIS_Data.point.lightning.flash[i].noise_index	= (unsigned char)(OTD_Data->Flashes[i].HDFFlash.QA[2]/10);
		LIS_Data.point.lightning.flash[i].glint_index	= (unsigned char)OTD_Data->Flashes[i].HDFFlash.QA[1];
		k = OTD_Data->Flashes[i].HDFFlash.QA[3];
		if (k > 255) k=255;
		LIS_Data.point.lightning.flash[i].density_index = k;
		k = (long)(LIS_Data.point.lightning.flash[i].TAI93_time-LIS_Data.one_second[0].TAI93_time);
		if ((k>=0) && (k<6100)) {
			LIS_Data.point.lightning.flash[i].alert_flag = LIS_Data.one_second[k].alert_summary;
		}
		/* These must be populated specially, some later... 
		LIS_Data.point.lightning.flash[i].oblong_index	=
		*/
#if defined(MAC) || defined(macintosh)	
#else
		LIS_Data.point.lightning.flash[i].grouping_sequence = OTD_Data->Flashes[i].HDFFlash.seq;
#endif
		LIS_Data.point.lightning.flash[i].grouping_status   = OTD_Data->Flashes[i].HDFFlash.end_status;
	}

	/* Areas */
	
	for (i=0; i<LIS_Data.point.point_summary.area_count; i++) {
		LIS_Data.point.lightning.area[i].TAI93_time 	= OTD_Data->Areas[i].HDFArea.TAI93;
		LIS_Data.point.lightning.area[i].delta_time 	= OTD_Data->Areas[i].HDFArea.delta;
		LIS_Data.point.lightning.area[i].observe_time 	= OTD_Data->Areas[i].HDFArea.view;
		LIS_Data.point.lightning.area[i].location[0]	= OTD_Data->Areas[i].HDFArea.cent[0];
		LIS_Data.point.lightning.area[i].location[1]	= OTD_Data->Areas[i].HDFArea.cent[1];
		LIS_Data.point.lightning.area[i].address		= i;
		LIS_Data.point.lightning.area[i].parent_address	= OTD_Data->Areas[i].HDFArea.orbit_id;
		LIS_Data.point.lightning.area[i].child_address	= OTD_Data->Areas[i].FirstFlash;
		LIS_Data.point.lightning.area[i].child_count	= OTD_Data->Areas[i].Flashes_Accepted;
#if defined(MAC) || defined(macintosh)	
#else
		LIS_Data.point.lightning.area[i].grouping_sequence = OTD_Data->Areas[i].HDFArea.seq;
#endif
		LIS_Data.point.lightning.area[i].grouping_status= OTD_Data->Areas[i].HDFArea.end_status;
		LIS_Data.point.lightning.area[i].approx_threshold = this_thr;
		LIS_Data.point.lightning.area[i].cluster_index	= (unsigned char)OTD_Data->Areas[i].HDFArea.QA[0];
		LIS_Data.point.lightning.area[i].noise_index	= (unsigned char)(OTD_Data->Areas[i].HDFArea.QA[2]/10);
		k = OTD_Data->Areas[i].HDFArea.QA[3];
		if (k > 255) k=255;
		LIS_Data.point.lightning.area[i].density_index	= k;
		k = (long)(LIS_Data.point.lightning.area[i].TAI93_time-LIS_Data.one_second[0].TAI93_time);
		if ((k>=0) && (k<6100)) {
			LIS_Data.point.lightning.area[i].alert_flag = LIS_Data.one_second[k].alert_summary;
		}
		/* These must be populated specially 
		LIS_Data.point.lightning.area[i].footprint		= 
		LIS_Data.point.lightning.area[i].oblong_index	=
		*/
	}	
	
	/* Viewtimes */
	
	for (i=0; i<LIS_Data.point.point_summary.vt_count; i++) {
		LIS_Data.point.viewtime[i].location[0] 		= OTD_Data->Orbits[0].ViewTimes[i].grid_point[0];
		LIS_Data.point.viewtime[i].location[1]		= OTD_Data->Orbits[0].ViewTimes[i].grid_point[1];
		LIS_Data.point.viewtime[i].TAI93_start		= OTD_Data->Orbits[0].ViewTimes[i].TAI93_start;
		LIS_Data.point.viewtime[i].TAI93_end		= OTD_Data->Orbits[0].ViewTimes[i].TAI93_start +
													  OTD_Data->Orbits[0].ViewTimes[i].duration;
		LIS_Data.point.viewtime[i].effective_obs	= OTD_Data->Orbits[0].ViewTimes[i].duration;
		LIS_Data.point.viewtime[i].approx_threshold = this_thr;
		for (j=(long)LIS_Data.point.viewtime[i].TAI93_start; j<=(long)LIS_Data.point.viewtime[i].TAI93_end; j++) {
			k = (long)(j-LIS_Data.one_second[0].TAI93_time);
			if ((k>=0) && (k<6100)) {
				LIS_Data.point.viewtime[i].alert_flag = 
					LIS_Data.point.viewtime[i].alert_flag | LIS_Data.one_second[k].alert_summary;
			}
		}
	}
	
	/* BG Summaries */
	
	for (i=0; i<LIS_Data.point.point_summary.bg_count; i++) {
		LIS_Data.point.bg_summary[i].TAI93_time		= OTD_Data->Orbits[0].Backgrounds[i].ImageAttribute.TAI93;
		LIS_Data.point.bg_summary[i].address		= i;
		LIS_Data.point.bg_summary[i].boresight[0]	= OTD_Data->Orbits[0].Backgrounds[i].ImageAttribute.nadir_location[0];
		LIS_Data.point.bg_summary[i].boresight[1] 	= OTD_Data->Orbits[0].Backgrounds[i].ImageAttribute.nadir_location[1];
		LIS_Data.point.bg_summary[i].corners[0]		= OTD_Data->Orbits[0].Backgrounds[i].ImageAttribute.NW_location[0];
		LIS_Data.point.bg_summary[i].corners[1]		= OTD_Data->Orbits[0].Backgrounds[i].ImageAttribute.NW_location[1];
		LIS_Data.point.bg_summary[i].corners[2]		= OTD_Data->Orbits[0].Backgrounds[i].ImageAttribute.NE_location[0];
		LIS_Data.point.bg_summary[i].corners[3]		= OTD_Data->Orbits[0].Backgrounds[i].ImageAttribute.NE_location[1];
		LIS_Data.point.bg_summary[i].corners[4]		= OTD_Data->Orbits[0].Backgrounds[i].ImageAttribute.SW_location[0];
		LIS_Data.point.bg_summary[i].corners[5]		= OTD_Data->Orbits[0].Backgrounds[i].ImageAttribute.SW_location[1];
		LIS_Data.point.bg_summary[i].corners[6]		= OTD_Data->Orbits[0].Backgrounds[i].ImageAttribute.SE_location[0];
		LIS_Data.point.bg_summary[i].corners[7]		= OTD_Data->Orbits[0].Backgrounds[i].ImageAttribute.SE_location[1];
	}
	

	
	/* Clean up descendant counts, area radiance */
	
	for (i=0; i < LIS_Data.point.point_summary.area_count; i++) {
	
		for (j=LIS_Data.point.lightning.area[i].child_address;
			 j<(LIS_Data.point.lightning.area[i].child_address+LIS_Data.point.lightning.area[i].child_count); j++) {
			 
			for (k=LIS_Data.point.lightning.flash[j].child_address;
				 k<(LIS_Data.point.lightning.flash[j].child_address+LIS_Data.point.lightning.flash[j].child_count); k++) {
			
				LIS_Data.point.lightning.flash[j].grandchild_count = 
					LIS_Data.point.lightning.flash[j].grandchild_count +
					LIS_Data.point.lightning.group[k].child_count;
					
			}
			
			LIS_Data.point.lightning.area[i].greatgrandchild_count =
				LIS_Data.point.lightning.area[i].greatgrandchild_count +
				LIS_Data.point.lightning.flash[j].grandchild_count;
			LIS_Data.point.lightning.area[i].grandchild_count =
				LIS_Data.point.lightning.flash[j].child_count;
			LIS_Data.point.lightning.area[i].net_radiance =
				LIS_Data.point.lightning.area[i].net_radiance +
				LIS_Data.point.lightning.flash[j].radiance;
				
		}
	
	}
	
	return(LIS_Data);
}

void   FreeData(TheData)
struct lis_orbit *TheData;
{   
    int orbits,  orb, im;
    
    free(TheData->point.lightning.area);
    free(TheData->point.lightning.flash);
    free(TheData->point.lightning.group);
    free(TheData->point.lightning.event);
    free(TheData->point.bg_summary);
    free(TheData->point.viewtime);
    free(TheData->one_second);
            
}

