/*
# include "hdf.h"
# include "lishdf_proto.h"
# include "lis_hdf.h"
# include "read_lishdf.h"
# include <math.h>
# include "mfhdf.h"
*/
# include "hdf.h"
/*
# include "lishdf_proto.h"
*/
# include "liblisotd_hdf.h"
# include "liblisotd_readhdf.h"
# include <math.h>
# include "mfhdf.h"

# define EARTH_RADIUS 	6370.0
# define TAI93_to_GPS	409881608.0
# define GPS_to_TAI93	-409881608.0
# define R2D		57.295779
# define D2R		.017453293


/* conversion of degrees */
#define  PI   3.14159265358979323846264338
#define RAD_TO_DEG ( (double)(180.0/PI) )
#define DEG_TO_RAD ( (double)(PI/180.0) )
#define RAD(x)     ((x) * DEG_TO_RAD)         /* degree to radian conversion */
#define RAD2DEG(x) ((x) * RAD_TO_DEG)         /* radiand to degrees */
#define Pi    3.14159265358979323846264338
#define Ae    RE_E
#define Be    RE_N
#define Fe   (( Ae - Be ) / Ae )
#define Ee   (sqrt( 2*Fe - Fe*Fe ))
#define EPS         ( (double) 1.0E-6 )
#define RE_E     ( ( double ) 6378137.0 )  /* Earth's axes WGS84 numbers */
#define RE_N     ( ( double ) 6356752.0 )

#define radians(x)  RAD(x)
#define degrees(x)  RAD2DEG(x)
#define Atan2(y,x)  atan2(y,x)

/* Valid polygons for spatial filtering */

#define	BOX	1
#define CIRCLE	2
#define CENTERED_BOX 3


/* Data hierarchy levels */

#define M       6
#define T	5
#define	A	1
#define F	2
#define G	3
#define E	4
#define	O	0

/* Valid WriteData() output types */

#define ASCII_SINGLE	0
#define ASCII_TREE	1
#define MINI_HDF	2
#define BINARY		3
#define SDS		6
#define METADATA	7

#define STDOUT		"stdout"
#define STDERR		"stderr"

#define ALL		T
#define GLOBAL		ALL
/*
#define AREA		A
#define FLASH		F
#define	GROUP		G
#define EVENT		E
*/

/* Preset levels for quality filtering */

#define PROB		0
#define PROB_BEST	100
#define PROB_GREEN	90
#define PROB_YELLOW	75
#define PROB_RED	50
#define PROB_WORST	0

#define GLINT		1
#define GLINT_BEST	180
#define GLINT_GREEN	21
#define GLINT_YELLOW	19
#define GLINT_RED	17
#define GLINT_WORST	15

#define	SNR		2
#define SNR_BEST	1000
#define SNR_GREEN	900
#define SNR_YELLOW	500
#define SNR_RED		100
#define SNR_WORST	0

#define TAC		3
#define TAC_BEST	32000
#define TAC_GREEN	140
#define TAC_YELLOW	120
#define TAC_RED		100
#define TAC_WORST	50

#define NUM_QUAL	4


/* Predefined thresholds for attribute filtering */

#define RADIANCE_MIN	0.0
#define RADIANCE_MAX	1e20
#define ENERGY_MIN	0
#define ENERGY_MAX	1e12
#define THRESH7_MIN	0
#define THRESH7_MAX      128
#define SIZE_MIN	0
#define SIZE_MAX	1e7
#define DURATION_MIN	0.0
#define DURATION_MAX	3.0
#define COVERAGE_MIN	0.0
#define COVERAGE_MAX	600.0
#define NUM_CHILDREN_MIN	1
#define NUM_CHILDREN_MAX	1e4

#define INDSTR		"     "
#define DEFAULT		"DEFAULT"
#define EMPTY		"EMPTY"

/* Valid ASCII output fields */

enum  data_parameters {
	TIME_UTC, LOCATION, POSITION, 
        SOLAR_ZENITH_ANGLE, DAY_NIGHT_TWILIGHT, D_N_T, DNT, ORBIT_NUM,
        NUM_CHILDREN, NUM_CHILDREN_ACCEPTED, NUM_CHILDREN_REJECTED,
        QUAL_PROB, QUAL_GLINT, QUAL_SNR, QUAL_TAC,
        TIME_TAI93, TIME_GPS, 
        QUAL_LIKELIHOOD,
        INDEX,
        VIEW_TIME, THRESH, RADIANCE, ENERGY, SIZE, DURATION, COVERAGE,
        THRESH7, RAD7
};
       
/* Valid parameters for attribute filtering are the subset of
      (RADIANCE, ENERGY, SIZE, DURATION, COVERAGE of the above  */



/* Metadata handling tags */

#define BROWSES			1
#define VIEWTIMES		2
#define IMAGES			3
#define HIRESBROWSES	4

#define IF_FLASHES		2
#define	KEEP			1
#define IGNORE			0

/* Smoothing window for platform error flags, sec */

#define SMOOTHING_WINDOW	300

/* Valid datum status for coincidence checking */

#define PENDING			0
#define OUT_OF_BOUNDS		1
#define NOT_SEEN		2
#define SEEN			3
#define NOT_FOUND		4
#define FOUND			5

/* Valid criteria for coincidence checking */

#define	TIME			0
#define SPACE			1

/* Valid times for coincidence checking */

#define FLASH_START_TIME	0
#define PEAK_GROUP_TIME		1

/* Valid parameters for loading point data files */

enum ascii_data_value {
    LATITUDE, LONGITUDE, 
    YEAR, MONTH, DAY, DOY, HOUR, MINUTE, SECOND,
    TAI93_TIME, GPS_TIME,
    DUMMY,
    INHERITED0, INHERITED1, INHERITED2, INHERITED3, INHERITED4, INHERITED5,
    ELEVATION
};

/* Type of XYT vertex/polygons */

#define TRACK 		1
#define PTOBS 		2

struct aTLEInfoSet {
	long	year,month,day;
};

struct aLatLonPair {
	double Latitude, Longitude;
};

struct anXYTtriplet{
	struct aLatLonPair	loc;
	double			time;
};

struct aUTCDateTime {
        int year, doy, month, day, hour, minute;
        double second;
};

struct aDateTime {
	struct	aUTCDateTime	UTC;
	struct  aUTCDateTime	Local;
	double	GPS, TAI93;
};

struct  aCoordArray {
    double c1[128][128], c2[128][128];
};

struct aquad {
	double c4,c3,c2,c1;
};

struct acal {
	struct aquad quad[4];
};

struct ametadatumstring	{
	char	val[132];
};

struct CBackground {
	struct 	otd_IMAGE_ATTRIBUTE		ImageAttribute;
	short int			*Image;
};

struct CImageAttribute	{
	struct otd_IMAGE_ATTRIBUTE		HDFImageAttribute;
	int				Use;
};

struct CViewTime {
	struct	otd_View_time 		HDFViewTime;
	int				Use;
};

struct CMetadata {
	char	metadata[256];
};
	
struct Orbit_summary {
	long	areas, flashes, groups, events, backgrounds;
	float	total_cal_radiance;
};

struct anEventRateVector {
	short int			Datum[6100];
};

struct anEventRateArray {
	struct anEventRateVector	Column[19];
};

struct aLongEventRateVector {
	long				Datum[6100];
};

struct aLongEventRateArray {
	struct aLongEventRateVector	Rate_Prefilter, Rate_Postglint, Rate_Postnoise,
					FIFO_full, Out_of_control, No_ephemeris, 
					Warmup, Connector_problem, No_ephemeris2,
					PlatformOK;
};

struct COrbit {
	struct  otd_Orbit_attributes	Attributes;
	struct	Orbit_summary		Summary;
	long				num_thresholds;
	struct	otd_THRESHOLD		*Thresholds;
	long				num_ephemeris;
	struct  otd_EPHEMERIS		*Ephemeris;
	double	TAI93_ephem_end;
	struct  anEventRateArray	Event_Rate;
	struct  aLongEventRateArray	Platform_Data;
	struct	otd_BROWSE			Browse[10368];
	struct  otd_DENSITY			Flash_Density_25[10368];
	struct  otd_DENSITY			Flash_Density_50[10368];
	long				num_backgrounds;
	struct  CBackground		*Backgrounds;
	long				num_viewtimes;
	struct	otd_View_time		*ViewTimes;
	int	*ViewTimeGrid;
	double  *ViewTimeGridStart, *ViewTimeGridEnd;
	long 	FirstArea, LastArea;
	long	Areas_Accepted, Areas_Rejected;
	long	NumChildren;
	char	filename[132];
	int	CoincFlag;
};

struct CSmallOrbit {
	struct  otd_Orbit_attributes	Attributes;
	struct  Orbit_summary		Summary;
	long	num_ephemeris;
	struct  otd_EPHEMERIS		*Ephemeris;
	double	TAI93_ephem_end;
	struct  anEventRateArray	Event_Rate;
	struct  aLongEventRateArray	Platform_Data;
	long 	FirstArea, LastArea;
	long	Areas_Accepted, Areas_Rejected;
	long	NumChildren;
	char	filename[132];
};
	
struct CArea {
	struct  otd_AREA		HDFArea;
	struct  aUTCDateTime	UTC;
	long			Orbit;
	long	Parent, FirstFlash, LastFlash;
	long	Flashes_Accepted, Flashes_Rejected;
	int	CoincFlag;
};

struct CFlash {
	struct	otd_FLASH		HDFFlash;
	struct  aUTCDateTime	UTC;
	long			Orbit;
	double  radiance, energy, size;
	long	Parent, FirstGroup, LastGroup;
	long    Groups_Accepted, Groups_Rejected;
	int	CoincFlag;
};

struct CGroup {
	struct 	otd_GROUP		HDFGroup;
	struct  aUTCDateTime	UTC;
	long			Orbit;
	double  radiance, energy;
	long	Parent, FirstEvent, LastEvent;
	long    Events_Accepted, Events_Rejected;
	int	CoincFlag;
};

struct CEvent {
	struct	otd_EVENT		HDFEvent;
	struct	aUTCDateTime	UTC;
	long			Orbit;
	double  radiance, energy;
	long 	Parent;
	int	CoincFlag;
};

struct anOTDDataset {
	struct CMetadata	*Metadata;
	struct COrbit		*Orbits;
	struct CArea		*Areas;
	struct CFlash		*Flashes;
	struct CGroup		*Groups;
	struct CEvent		*Events;
	struct aTimeWindow	*Gaps;
	long	num_orbits, num_areas, num_flashes,
		num_groups, num_events, num_metadata;
};

struct  CoincResults {
	int	num_out_of_bounds;
	int 	num_not_seen;
	int	num_seen;
	int	num_not_found;
	int	num_found;
};

struct	aPointDatum {
	struct	aDateTime	DateTime;
	float			Position[2];
	double			Inherited[5];
	int			Status;
	int			Orbit;
	double			Time_error, Range_error, Azimuth;
	long			OTD_datum_index;
};

struct	aPointDataset {
	struct CoincResults	Results;
	long			num_point_data;
	struct aPointDatum	*Data;
};

struct	aHadFieldSet	 {
	int		has_year, has_month, has_day, has_doy,
			has_hour, has_minute, has_second, 
			has_TAI93, has_GPS, has_latitude, has_longitude,
			has_ymd, has_hms, has_fulltime,
			has_fixed_year, has_fixed_month, has_fixed_day,
			has_fixed_doy;
	int		fixed_year, fixed_month, fixed_day, fixed_doy;
};

struct  anASCIIDataFileAttributeSet {
	int		num_cols;
	double	        cols[20];
	enum ascii_data_value	content[20];
	struct	aHadFieldSet	HadFields;
};

struct	aCoincParamset {
	int	time_to_use;
	int	crit_to_use;
	double	range_error_km, time_error_sec;
	int	data_level;
	double  max_range_km;
	double  min_viewtime;
};

struct  anXYTDatum {
	struct	aDateTime	DateTime;
	float				Position[2];
	double 				Inherited[5];
	float				Elevation;
};

struct	anXYTDataset {
	long	num_raw, num_interp;
	struct	aHadFieldSet	HadFields;
	struct	anXYTDatum	*Raw;
	struct	anXYTDatum	*Interp;
	char    filename[132];
};

struct aTimeWindow {
	int	Active;
	struct	aDateTime	Start, End;
};

struct	aBoxBound {
	struct  aLatLonPair	LL, UR;
};

struct  aCircleBound {
	struct  aLatLonPair	Center;
	double	Min_Range_km, Max_Range_km;
};

struct anAreaBound {
	int 	Active;
	int	BoundType;
        union   aBound {
		struct aBoxBound	Box;
		struct aCircleBound	Circle;
	} Bounds;
};

struct aDoubleLoHi	{
	double 	lo,hi;
};

struct anIntLoHi {
	int lo, hi;
};

struct aLevelQualityBound {
	struct anIntLoHi	Param[NUM_QUAL];
};

struct aQualityBound {
	int	Active;
	struct	aLevelQualityBound	Level[6];
};

struct aDataAttributeBound {
	struct aDoubleLoHi	radiance, energy, area, duration,coverage;
	struct anIntLoHi	num_children, thresh;
};

struct aDataAttributeBoundSet {
	int	Active;
	int	GlobalActive, AreaActive, FlashActive, GroupActive, EventActive;
	struct 	aDataAttributeBound	
				Global, Area, Flash, Group, Event;
};

struct	anXYTBound {
	int	   Active;
	struct anXYTDataset	*XYT;
	int	   XYT_bound_type;
	float	max_range_km, min_range_km;
	float	min_elev, max_elev;
	float   timeslop_sec;
	float   timeslopres_sec;
};

struct anOTDBoundset {
	struct aTimeWindow				TimeWindow;
	struct anAreaBound				AreaBound;
	struct anXYTBound				XYTBound;
	struct aQualityBound			QualityBound;
	struct aDataAttributeBoundSet	AttributeBounds;
};

struct aBoundset {
	struct aTimeWindow				TimeWindow;
	struct anAreaBound				AreaBound;
	struct anXYTBound				XYTBound;
	struct aQualityBound			QualityBound;
	struct aDataAttributeBoundSet	AttributeBounds;
};

typedef union aFileID {
	char	name[8][132];
	FILE    *out_stream[8];
	long    hdf_id;
};

struct	anHDFFileEntry {
	long	hdffile;
	char	hdfname[132];
};

struct aWriteParam {
	enum    data_parameters Var;
	char	CFormat[8];
};

struct aWriteParamSet {
	int    NumParams;
	int    UseOrNot;
	struct aWriteParam Param[20];
};

struct theWriteParams {
	struct aWriteParamSet	Area, Flash, Group, Event;
};

struct theSDSFields {
	int 	HiresBrowse, Browse, Viewtime, Images;
};

struct theMetadataHandling {
	int 	Browses, Viewtimes, Images;
};

struct anSDS {
	float32 *grid;
	float32 *latptr, *lonptr;
	int32   dims[2];
};

struct outputSettings {
	int	   							DoASCIISingle, DoASCIITree, DoSDS, DoMetadata, DoMiniHDF;
	struct theWriteParams			TheASCIIWriteParams;
	struct theSDSFields				TheSDSFields;
	struct anAreaBound				TheSDSBounds;
};
	
struct anOTDPrefset {
	struct anOTDBoundset			TheBounds;
	struct theMetadataHandling		TheMetadataHandling;
	struct outputSettings			TheOutputSettings;
};


double			Cflash_rad(struct anOTDDataset *TheData, struct CFlash thisFlash);
double			Cgroup_rad(struct anOTDDataset *TheData, struct CGroup thisGroup);
double			Cmax_stroke_area(struct anOTDDataset TheData, struct CFlash thisFlash);
double			approx_flash_area(struct otd_FLASH flash);
double			energy_to_power(double energy, double pulsewidth);
double			flash_rad(struct otd_FLASH thisFlash);
double			group_rad(struct otd_GROUP thisGroup);
double			max_stroke_area(struct otd_FLASH flash);
double			peak_group_time(long flash_num, struct anOTDDataset *Data, double *pgr);
double 			event_rad(struct otd_EVENT thisEvent);
double 			rad_to_energy(double radiance);
double 			total_flash_area(struct otd_FLASH flash);
double  		UTC_to_GPS(struct aUTCDateTime UTC);
double  		UTC_to_TAI93(struct aUTCDateTime UTC);
double  		UTC_to_TAI93b(struct aUTCDateTime UTC);
FILE			*OpenStream(char filename[132]);
int				getDayOfYear(int dy, int mo, int yr);
int				GetOrbitByTime(struct anOTDDataset *OTD, double TAI93);
int				AreaOK( struct otd_AREA area, struct anOTDBoundset *bounds); 
int				CheckLocation(float latlon[2], struct anOTDBoundset *bounds);
int				CheckXYT(double lat, double lon, double TAI93, struct anOTDBoundset *Bounds);
int				EventOK(struct otd_EVENT event, struct anOTDBoundset *bounds);
int				FlashOK(struct otd_FLASH flash, struct anOTDBoundset *bounds);
int				GroupOK(struct otd_GROUP group, struct anOTDBoundset *bounds);
int				OrbitOK(struct otd_Orbit_attributes orbit, struct anOTDBoundset *bounds);
int				PlatformOK(double start, double end, struct otd_Orbit_attributes orb_att, struct aLongEventRateArray *plat_data);
int				TestQuality(int level, short int qa[4], struct anOTDBoundset *bounds, int pf[NUM_QUAL]);
int				getquad(int x, int y);
int				CheckCoinc(struct anXYTtriplet Point, struct anXYTtriplet OTD, 
				struct aCoincParamset Params, double *delt, double *delr, double *az);
long			LatLon2Grid(double latitude, double longitude, double resolution);
long			getDayNum(int day, int month, int year);
long			read_orbit_summary_struct(struct Orbit_summary *sum, long record);
long			read_metadata_struct(char sum[128], long record);
void			GetFileSize(FILE *tmpfile, long *nl);
struct  aUTCDateTime	GPS_to_UTC(double GPS);
struct  aUTCDateTime	TAI93_to_UTC(double TAI93);
struct  aUTCDateTime	TAI93_to_UTCb(double TAI93);
struct  anOTDDataset	GetData(char hdfname[], struct anOTDBoundset *Bounds);
struct  anOTDDataset	GetPointCoincidence(struct anOTDDataset *OTD_Data,
					    struct aPointDataset *Point_Data,
					    struct anOTDBoundset *Bounds,
					    struct aCoincParamset Params);
union 	aFileID 	OpenFileID(int output_type, char filename[132], int dump_flags[5]);
void			AddASCIIOutputField(int type, enum data_parameters var, char format[8]);
void			AddSDSOutputField(int type);
void			Metadata2Buf(struct CMetadata *MD, unsigned char buf[128]);
void			Area2Buf(struct otd_AREA *area, unsigned char buf[128]);
void			Attribute2Buf(struct otd_Orbit_attributes *attribute, unsigned char buf[128]);
void			Browse2Buf(struct otd_BROWSE *browse, unsigned char buf[128]);
void			ChangeSDSOutputField(int type, int setting);
void 			CloseFileID(union aFileID OutID, int output_type, int dump_flags[5]);
void			CloseStream(FILE *outfile);
void			Ephem2Buf(struct otd_EPHEMERIS *ephem, unsigned char buf[128]);	
void			Event2Buf(struct otd_EVENT *event, unsigned char buf[128]);
void			Flash2Buf(struct otd_FLASH *flash, unsigned char buf[128]);
void			FreeOTDData(struct anOTDDataset *TheData);
void			getDate(long daynum, int *year, int *month, int *day, int *doy);
void			Grid2Buf(struct otd_DENSITY *grid, unsigned char buf[128]);
void			Grid2LatLon(long idx, double res, double *lat, double *lon);
void			Group2Buf(struct otd_GROUP *group, unsigned char buf[128]);
void			ImageAttribute2Buf(struct otd_IMAGE_ATTRIBUTE *imatt, unsigned char buf[128]);
void 			Initialize();
void			InitBrowse(long orbits_found, struct CSmallOrbit *TempOrbits, struct anOTDDataset *TheData);
void			InvJulian(int year, int doy, int *month, int *day);
void			LatLons2RgAz(double lat0, double lon0, double lat1, double lon1, double *az0, double *az1, double *rg);
void			LoadPlatformData(long oa, long orbits_found, struct CSmallOrbit *TempOrbits, struct anOTDDataset *TheData);
void			ProcessViewTimes(long first_new_orbit, long orbits_found, long total_viewtime, 
					 struct CViewTime *TempViewTime, struct anOTDDataset *TheData, 
					 struct anOTDBoundset *Bounds, long flashes_found);
void			ProcessThresholds(long first_new_orbit, long orbits_found, long total_thresh,
					 struct anOTDDataset *TheData);
void			ProcessBackgrounds(long first_new_orbit, long orbits_found, long total_images,
					 struct CImageAttribute *TempImageAttributes, struct anOTDDataset *TheData,long flashes_found);
void			Rate2Buf(struct anEventRateVector *rate, unsigned char erbuf[12200]);			
void			RemoveSDSOutputField(int type);
void			ResetASCIIOutputFields(int type, char reset_value[8]);
void			ResetAllBounds(struct anOTDBoundset *bounds);
void			ResetAreaBound( struct anOTDBoundset *bounds);
void			ResetXYTBound(struct anOTDBoundset *bounds);
void			ResetDataAttributeBound(struct anOTDBoundset *bounds);
void			ResetMetadataHandling();
void			ResetQualityBound(struct anOTDBoundset *bounds);
void			ResetSDSOutputFields();
void			ResetTimeWindow(struct anOTDBoundset *bounds);
void			RgAz2LatLon(double  dglat1, double  dglon1, double *dglat2, double *dglon2, double  dfaz, double  *baz, double s);
void			SetAreaBound(struct anOTDBoundset *bounds,
				     int type, double par1, double par2, 
			             double par3, double par4);
void			SetXYTBound(struct anOTDBoundset *Bounds, int type, struct anXYTDataset *XYTData, double par1,
							double par2, double par3, double par4);
void			SetDataAttributeBound(struct anOTDBoundset *bounds,
					int level, enum data_parameters type, double lower, double upper);
void			SetMetadataHandling(int type, int setting);
void			SetOutFilePath(char *filepath);
void		        SetQualityBound(struct anOTDBoundset *bounds,
					int level, int type, int lower, int upper);
void			SetRawFilePath(char *filepath);
void			SetTimeWindow_YMD(struct anOTDBoundset *bounds,
					  int syr, int smo, int sdy, int shr,
					  int smi, double sse, int eyr, int emo,
					  int edy, int ehr, int emi, double ese);
void			SetTimeWindow_JD(struct anOTDBoundset *bounds,
					 int syr, int sdoy, int shr,
					 int smi, double sse,
					 int eyr, int edoy, int ehr, 
					 int emi, double ese);
void			Summary2Buf(struct Orbit_summary *sum, unsigned char buf[128]);
void			Threshold2Buf(struct otd_THRESHOLD *thresh, unsigned char buf[128]);
void			ViewTime2Buf(struct otd_View_time *vt, unsigned char buf[128]);
void			WriteSDS(struct anOTDDataset TheData, char filename[132]);
void			Write_Area_ASCII(FILE *outfile, char indstr[132], long idx, struct anOTDDataset TheData);
void			Write_Binary_Browse(struct anOTDDataset TheData, int type, FILE *outfile, long num_elements, long num_gaps);
void			Write_Binary_Data(FILE *outfile, long num_elements, long *binary_data);
void			Write_Binary_FData(FILE *outfile, long num_elements, float *binary_data);
void			Write_Binary_Header1(struct anOTDDataset TheData, FILE *outfile, long num_granules, long total_elements, long total_gaps);
void			Write_Binary_Header2(struct anOTDDataset TheData, FILE *outfile, long num_elements, char time_format[8], long num_gaps);
void			Write_Event_ASCII(FILE *outfile, char indstr[132], long idx, struct anOTDDataset TheData);
void			Write_Flash_ASCII(FILE *outfile, char indstr[132], long idx, struct anOTDDataset TheData);
void			Write_Group_ASCII(FILE *outfile, char indstr[132], long idx, struct anOTDDataset TheData);
void			Write_Mini_HDF(struct anOTDDataset TheData, int output_type, char filename[132], int dump_or, int dump_ar, int dump_fl, int dump_gr, int dump_ev);
void			Write_Orbit_ASCII(FILE *outfile, char indstr[132], long idx, struct anOTDDataset TheData);
void   			WriteData(struct anOTDDataset TheData, int output_type, char filename[132], int dump_or, int dump_ar, int dump_fl, int dump_gr, int dump_ev);
void 			SetASCIIDataFileColumn(struct anASCIIDataFileAttributeSet *PDFInfo, int col, enum ascii_data_value param);
void			FixASCIIDataFileValue(struct anASCIIDataFileAttributeSet *PDFInfo, enum ascii_data_value param, int value);
void			CompressData(struct anOTDDataset *Data);
void			LoadXYTDataFile(char filename[128], struct anXYTDataset *XYTData, struct anASCIIDataFileAttributeSet DFInfo);
void 			LoadPointDataFile(char filename[128], struct aPointDataset *PointData, struct anASCIIDataFileAttributeSet DFInfo, struct anOTDDataset *OTDData, struct anOTDBoundset *Bounds);
float32		    *SubArray(float32 *old, int32 d1, int32 d2, int32 min_x, int32 min_y, int32 max_x, int32 max_y);
void			ReduceSDS(struct anSDS *old, struct anSDS *new);
void			SetSDSBounds(int type, double p1, double p2, double p3, double p4);
int				GetThreshold(struct anOTDDataset *TheData, long orbit, double TAI93);
int 			LoadSatelliteXYT(char reqsat[20],double StartTAI,double StopTAI,int delta, int suntype,
				struct anXYTDataset *Track);
void 			GetTimeBounds(char hdfname[132],struct aDateTime *Start, struct aDateTime *Stop);
void			pickTleFile(double TAI);
void			SetTLEFilePath(char path[132]);
void			AddMetadataLine(struct anOTDDataset *Data,char Line[256]);
void			AddMetadataCount(struct anOTDDataset *Data,long o,
  				long a, long f, long g, long e, long i, long v);
void			AddMetadataBounds(struct anOTDDataset *Data, struct anOTDBoundset *Bounds);
void			WriteMetadata(struct anOTDDataset *TheData, FILE *outfile);
int			XYTInterp(struct anXYTDataset *XYT, double TAI93, struct anXYTDatum *Out);
double			fromcalcurves(int count, int callevel, int q);
double			DC_12bit_to_rad(long count, int x, int y);
long 			SZA_to_12bit(short int sza);
float 			SZA_to_rad(short int sza);
float 			cosSZA_to_rad(float cossza);
double 			event_cal(short int count7, int x, int y, float bgr);
double			CEvent_cal(struct CEvent anevent);
double		        HDFEvent_cal(struct otd_EVENT anevent);

/*
#define	TIME_UTC	100
#define LOCATION	101
#define POSITION	101
#define SOLAR_ZENITH_ANGLE	102
#define DAY_NIGHT_TWILIGHT	103
#define ORBIT_NUM		104
#define NUM_CHILDREN_ACCEPTED	105
#define NUM_CHILDREN_REJECTED	106
#define	QUAL_PROB		107
#define QUAL_GLINT		108
#define QUAL_SNR		109
#define QUAL_TAC		110
#define	TIME_TAI93		111
#define TIME_GPS		112
#define QUAL_LIKELIHOOD		113
#define INDEX			114
#define VIEW_TIME		115
#define D_N_T			116
#define DNT			116
#define	THRESH		117
*/

/*
#define LATITUDE	1
#define LONGITUDE	2
#define	YEAR		3
#define MONTH		4
#define DAY			5
#define DOY			6
#define HOUR		7
#define MINUTE		8
#define SECOND		9
#define TAI93_TIME	10
#define GPS_TIME	11
#define DUMMY		12
#define INHERITED0	13
#define INHERITED1	14
#define	INHERITED2	15
#define INHERITED3	16
#define INHERITED4	17
#define INHERITED5	18
#define ELEVATION	19
*/

/*
#define	RADIANCE	0
#define	ENERGY		1
#define SIZE		2
#define DURATION	3
#define COVERAGE	4
#define NUM_CHILDREN	5
*/
